// Include the MIDI library
#include <MIDI.h>
#include "Adafruit_seesaw.h"
#include <seesaw_neopixel.h>

#define  DEFAULT_I2C_ADDR 0x30
#define  ANALOGIN   18
#define  NEOPIXELOUT 9

Adafruit_seesaw seesaw;
Adafruit_seesaw seesaw2;
seesaw_NeoPixel pixels = seesaw_NeoPixel(4, NEOPIXELOUT, NEO_GRB + NEO_KHZ800);
seesaw_NeoPixel pixels2 = seesaw_NeoPixel(4, NEOPIXELOUT, NEO_GRB + NEO_KHZ800);

// Create and bind the MIDI interface to the default hardware Serial port
MIDI_CREATE_DEFAULT_INSTANCE();

// Multiplexer 1 variables
int r0 = 0;
int r1 = 0;     
int r2 = 0;      
int s0 = 3;
int s1 = 4;
int s2 = 5;

// Multiplexer 2 variables
int m0 = 0;
int m1 = 0;     
int m2 = 0;      
int n0 = 8;
int n1 = 7;
int n2 = 6;

// Multiplexer 3 variables
int p0 = 0;
int p1 = 0;     
int p2 = 0;      
int q0 = 0;
int q1 = 1;
int q2 = 2;

int count = 0;   //which y pin we are selecting

uint16_t slide_val = 0;
uint16_t old_slide_val = 0;
uint16_t slide2_val = 0;
uint16_t old_slide2_val = 0;

// on/off variables for 8 steps
const int numKeys = 8; // number of steps for sequencer
int dialValues1[numKeys] = {60, 60, 60, 60, 60, 60, 60, 60};
int dialValues2[numKeys] = {60, 60, 60, 60, 60, 60, 60, 60};
int dialValues3[numKeys] = {60, 60, 60, 60, 60, 60, 60, 60};
int dialValues4[numKeys] = {60, 60, 60, 60, 60, 60, 60, 60};
int oldDialValues1[numKeys] = {1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024};
int oldDialValues3[numKeys] = {1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024};
int lights[numKeys] = {LOW, LOW, LOW, LOW, LOW, LOW, LOW, LOW};
int step = 0;
int duration = 50;
int sensorReading = 0;
int tempoReading = 0;
int newTempoReading = 0;
int tuneReading = 0;
int newTuneReading = 0;
int dialController = 0;
int sendDialValue = 0;
int sendSlideValue = 0;
int lightCount = 0;

int ledState = LOW;  // ledState used to set the LED
unsigned long previousMillis = 0;  // will store last time LED was updated
const long interval = 500;

void setup(){  

  pinMode(s0, OUTPUT);
  pinMode(s1, OUTPUT);
  pinMode(s2, OUTPUT);

  pinMode(n0, OUTPUT);
  pinMode(n1, OUTPUT);
  pinMode(n2, OUTPUT);

  pinMode(q0, OUTPUT);
  pinMode(q1, OUTPUT);
  pinMode(q2, OUTPUT);

  pinMode(A4, OUTPUT);

  pinMode(A6, INPUT);

  MIDI.begin();


  Serial.begin(31250);

  while (!Serial) delay(10);   // wait until serial port is opened

  if (!seesaw.begin(DEFAULT_I2C_ADDR) || !seesaw2.begin(DEFAULT_I2C_ADDR+1)) {
  Serial.println(F("seesaws not found!"));
  while(1) delay(10);
  }

}

 

void loop () {

  // // reset?
  // if (readButton(A6, 10)) {
  //   resetAll();
  // }

  updateValues1();
  updateValues2();
  updateValues3();
  updateValues4();

  dialController = 0;
  sendDialValue = 0;

  // Serial.print("MIDI input: ");
  // Serial.println(MIDI.read());

  if (MIDI.read()) {
    Serial.println("MIDI input received");
    for(int i = 0; i < numKeys; i++) {
      if (i == lightCount) {
        lights[i] = HIGH;
      } else {
        lights[i] = LOW;
      }
    }
    updateValues5();
    lightCount++;
  } 
  if (lightCount == 8) {
    lightCount = 0;
  }

  // send updated values for knobs 1-8
  for (int k = 0; k <= 7; k++) {
    // check if dial has been changed, and if it has, send an update to MIDI controller
    if (dialValues2[k] != dialValues1[k]) {
      sendDialValue = dialValues1[k];
      MIDI.sendControlChange(dialController, sendDialValue, 2);
      dialValues2[k] = dialValues1[k];
      resetAll();
      printValues();
    }
    dialController++;
  }


  for (int k = 0; k <= 7; k++) {
    // check if dial has been changed, and if it has, send an update to MIDI controller
    if (dialValues4[k] != dialValues3[k]) {
      sendDialValue = dialValues3[k];
      MIDI.sendControlChange(dialController, sendDialValue, 2);
      dialValues4[k] = dialValues3[k];
      resetAll();
      printValues();
    }
    dialController++;
  }
  
  if (abs(old_slide_val - slide_val) > 15) {
    if (old_slide_val != slide_val) {
      sendSlideValue = slide_val;
      if (sendSlideValue < 256) {
        MIDI.sendControlChange(18, 0, 2);
      } else if (sendSlideValue < 512) {
        MIDI.sendControlChange(18, 42, 2);
      } else if (sendSlideValue < 768) {
        MIDI.sendControlChange(18, 84, 2);
      } else if (sendSlideValue < 1024) {
        MIDI.sendControlChange(18, 127, 2);
      }
      old_slide_val = slide_val;
      resetAll();
      printValues();
    }
  }

  if (abs(old_slide2_val - slide2_val) > 15) {
    if (old_slide_val != slide2_val) {
      sendSlideValue = slide2_val;
        if (sendSlideValue < 256) {
        MIDI.sendControlChange(19, 0, 2);
      } else if (sendSlideValue < 512) {
        MIDI.sendControlChange(19, 42, 2);
      } else if (sendSlideValue < 768) {
        MIDI.sendControlChange(19, 84, 2);
      } else if (sendSlideValue < 1024) {
        MIDI.sendControlChange(19, 127, 2);
      }
      old_slide2_val = slide2_val;
      resetAll();
      printValues();
    }
  }

  dialController = 0;
}

void updateValues1() {
  
  for (count=0; count<=7; count++) {

    // select the bit  
    r0 = bitRead(count,0);
    r1 = bitRead(count,1);  
    r2 = bitRead(count,2);    

    // form connection to right input on multiplexer
    digitalWrite(s0, r0);
    digitalWrite(s1, r1);
    digitalWrite(s2, r2); 

    sensorReading = analogRead(A0);

    // if the difference isn't just noise, then print off the new dial values
    if(abs(oldDialValues1[count]-sensorReading) > 15) {
      
      // save the new different value as the old value
      oldDialValues1[count] = sensorReading;
    
      // if a dial has been turned (noise greater than 5), then set the dialValues1[count] to analogRead(A0) and print the new array
      if(sensorReading < 41) {
        dialValues1[count] = 72;
      } else if (sensorReading < 82) {
        dialValues1[count] = 71;
      } else if (sensorReading < 123) {
        dialValues1[count] = 70;
      } else if (sensorReading < 164) {
        dialValues1[count] = 69;
      } else if (sensorReading < 205) {
        dialValues1[count] = 68;
      } else if (sensorReading < 246) {
        dialValues1[count] = 67;
      } else if (sensorReading < 287) {
        dialValues1[count] = 66;
      } else if (sensorReading < 328) {
        dialValues1[count] = 65;
      } else if (sensorReading < 369) {
        dialValues1[count] = 64;
      } else if (sensorReading < 410) {
        dialValues1[count] = 63;
      } else if (sensorReading < 451) {
        dialValues1[count] = 62;
      } else if (sensorReading < 492) {
        dialValues1[count] = 61;
      } else if (sensorReading < 533) {
        dialValues1[count] = 60;
      } else if (sensorReading < 574) {
        dialValues1[count] = 59;
      } else if (sensorReading < 615) {
        dialValues1[count] = 58;
      } else if (sensorReading < 656) {
        dialValues1[count] = 57;
      } else if (sensorReading < 697) {
        dialValues1[count] = 56;
      } else if (sensorReading < 738) {
        dialValues1[count] = 55;
      } else if (sensorReading < 779) {
        dialValues1[count] = 54;
      } else if (sensorReading < 820) {
        dialValues1[count] = 53;
      } else if (sensorReading < 861) {
        dialValues1[count] = 52;
      } else if (sensorReading < 902) {
        dialValues1[count] = 51;
      } else if (sensorReading < 943) {
        dialValues1[count] = 50;
      } else if (sensorReading < 984) {
        dialValues1[count] = 49;
      } else if (sensorReading < 1024) {
        dialValues1[count] = 48;
      } 

    }
  }  
}

void updateValues2() {
  
  for (count=0; count<=7; count++) {

    // select the bit  
    m0 = bitRead(count,0);
    m1 = bitRead(count,1);  
    m2 = bitRead(count,2);    

    // form connection to right input on multiplexer
    digitalWrite(n0, m0);
    digitalWrite(n1, m1);
    digitalWrite(n2, m2); 

    sensorReading = analogRead(A1);

    // if the difference isn't just noise, then print off the new dial values
    if(abs(oldDialValues3[count]-sensorReading) > 15) {
      
      // save the new different value as the old value
      oldDialValues3[count] = sensorReading;
    
      // if a dial has been turned (noise greater than 5), then set the dialValues3[count] to analogRead(A0) and print the new array
      if(sensorReading < 41) {
        dialValues3[count] = 72;
      } else if (sensorReading < 82) {
        dialValues3[count] = 71;
      } else if (sensorReading < 123) {
        dialValues3[count] = 70;
      } else if (sensorReading < 164) {
        dialValues3[count] = 69;
      } else if (sensorReading < 205) {
        dialValues3[count] = 68;
      } else if (sensorReading < 246) {
        dialValues3[count] = 67;
      } else if (sensorReading < 287) {
        dialValues3[count] = 66;
      } else if (sensorReading < 328) {
        dialValues3[count] = 65;
      } else if (sensorReading < 369) {
        dialValues3[count] = 64;
      } else if (sensorReading < 410) {
        dialValues3[count] = 63;
      } else if (sensorReading < 451) {
        dialValues3[count] = 62;
      } else if (sensorReading < 492) {
        dialValues3[count] = 61;
      } else if (sensorReading < 533) {
        dialValues3[count] = 60;
      } else if (sensorReading < 574) {
        dialValues3[count] = 59;
      } else if (sensorReading < 615) {
        dialValues3[count] = 58;
      } else if (sensorReading < 656) {
        dialValues3[count] = 57;
      } else if (sensorReading < 697) {
        dialValues3[count] = 56;
      } else if (sensorReading < 738) {
        dialValues3[count] = 55;
      } else if (sensorReading < 779) {
        dialValues3[count] = 54;
      } else if (sensorReading < 820) {
        dialValues3[count] = 53;
      } else if (sensorReading < 861) {
        dialValues3[count] = 52;
      } else if (sensorReading < 902) {
        dialValues3[count] = 51;
      } else if (sensorReading < 943) {
        dialValues3[count] = 50;
      } else if (sensorReading < 984) {
        dialValues3[count] = 49;
      } else if (sensorReading < 1024) {
        dialValues3[count] = 48;
      }    

    }
  }  
}

void updateValues3() {
  
  newTempoReading = analogRead(A2);
  newTuneReading = analogRead(A3);

  // if the difference isn't just noise, then print off the new dial values
  if(abs(newTempoReading-tempoReading) > 25) {
    
    // save the new different value as the old value
    tempoReading = newTempoReading;
  
    // if a dial has been turned (noise greater than 5), then set the dialValues3[count] to analogRead(A0) and print the new array
    if(tempoReading < 41) {
      MIDI.sendControlChange (16, 72, 2);
    } else if (tempoReading < 82) {
      MIDI.sendControlChange (16, 71, 2);
    } else if (tempoReading < 123) {
      MIDI.sendControlChange (16, 70, 2);
    } else if (tempoReading < 164) {
      MIDI.sendControlChange (16, 69, 2);
    } else if (tempoReading < 205) {
      MIDI.sendControlChange (16, 68, 2);
    } else if (tempoReading < 246) {
      MIDI.sendControlChange (16, 67, 2);
    } else if (tempoReading < 287) {
      MIDI.sendControlChange (16, 66, 2);
    } else if (tempoReading < 328) {
      MIDI.sendControlChange (16, 65, 2);
    } else if (tempoReading < 369) {
      MIDI.sendControlChange (16, 64, 2);
    } else if (tempoReading < 410) {
      MIDI.sendControlChange (16, 63, 2);
    } else if (tempoReading < 451) {
      MIDI.sendControlChange (16, 62, 2);
    } else if (tempoReading < 492) {
      MIDI.sendControlChange (16, 61, 2);
    } else if (tempoReading < 533) {
      MIDI.sendControlChange (16, 60, 2);
    } else if (tempoReading < 574) {
      MIDI.sendControlChange (16, 59, 2);
    } else if (tempoReading < 615) {
      MIDI.sendControlChange (16, 58, 2);
    } else if (tempoReading < 656) {
      MIDI.sendControlChange (16, 57, 2);
    } else if (tempoReading < 697) {
      MIDI.sendControlChange (16, 56, 2);
    } else if (tempoReading < 738) {
      MIDI.sendControlChange (16, 55, 2);
    } else if (tempoReading < 779) {
      MIDI.sendControlChange (16, 54, 2);
    } else if (tempoReading < 820) {
      MIDI.sendControlChange (16, 53, 2);
    } else if (tempoReading < 861) {
      MIDI.sendControlChange (16, 52, 2);
    } else if (tempoReading < 902) {
      MIDI.sendControlChange (16, 51, 2);
    } else if (tempoReading < 943) {
      MIDI.sendControlChange (16, 50, 2);
    } else if (tempoReading < 984) {
      MIDI.sendControlChange (16, 49, 2);
    } else if (tempoReading < 1024) {
      MIDI.sendControlChange (16, 48, 2);
    }    
    resetAll();
    printValues();
  }

    // if the difference isn't just noise, then print off the new dial values
  if(abs(newTuneReading-tuneReading) > 25) {
    
    // save the new different value as the old value
    tuneReading = newTuneReading;
  
    // if a dial has been turned (noise greater than 5), then set the dialValues3[count] to analogRead(A0) and print the new array
    if(tuneReading < 41) {
      MIDI.sendControlChange (17, 72, 2);
    } else if (tuneReading < 82) {
      MIDI.sendControlChange (17, 71, 2);
    } else if (tuneReading < 123) {
      MIDI.sendControlChange (17, 70, 2);
    } else if (tuneReading < 164) {
      MIDI.sendControlChange (17, 69, 2);
    } else if (tuneReading < 205) {
      MIDI.sendControlChange (17, 68, 2);
    } else if (tuneReading < 246) {
      MIDI.sendControlChange (17, 67, 2);
    } else if (tuneReading < 287) {
      MIDI.sendControlChange (17, 66, 2);
    } else if (tuneReading < 328) {
      MIDI.sendControlChange (17, 65, 2);
    } else if (tuneReading < 369) {
      MIDI.sendControlChange (17, 64, 2);
    } else if (tuneReading < 410) {
      MIDI.sendControlChange (17, 63, 2);
    } else if (tuneReading < 451) {
      MIDI.sendControlChange (17, 62, 2);
    } else if (tuneReading < 492) {
      MIDI.sendControlChange (17, 61, 2);
    } else if (tuneReading < 533) {
      MIDI.sendControlChange (17, 60, 2);
    } else if (tuneReading < 574) {
      MIDI.sendControlChange (17, 59, 2);
    } else if (tuneReading < 615) {
      MIDI.sendControlChange (17, 58, 2);
    } else if (tuneReading < 656) {
      MIDI.sendControlChange (17, 57, 2);
    } else if (tuneReading < 697) {
      MIDI.sendControlChange (17, 56, 2);
    } else if (tuneReading < 738) {
      MIDI.sendControlChange (17, 55, 2);
    } else if (tuneReading < 779) {
      MIDI.sendControlChange (17, 54, 2);
    } else if (tuneReading < 820) {
      MIDI.sendControlChange (17, 53, 2);
    } else if (tuneReading < 861) {
      MIDI.sendControlChange (17, 52, 2);
    } else if (tuneReading < 902) {
      MIDI.sendControlChange (17, 51, 2);
    } else if (tuneReading < 943) {
      MIDI.sendControlChange (17, 50, 2);
    } else if (tuneReading < 984) {
      MIDI.sendControlChange (17, 49, 2);
    } else if (tuneReading < 1024) {
      MIDI.sendControlChange (17, 48, 2);
    }    
    resetAll();
    printValues();
  }
}

void updateValues4() {
  slide_val = seesaw.analogRead(ANALOGIN);
  slide2_val = seesaw2.analogRead(ANALOGIN);
}

void updateValues5() {
  
  for (count=0; count<=7; count++) {

    // select the bit  
    p0 = bitRead(count,0);
    p1 = bitRead(count,1);  
    p2 = bitRead(count,2);    

    // form connection to right input on multiplexer
    digitalWrite(q0, p0);
    digitalWrite(q1, p1);
    digitalWrite(q2, p2); 


    if (lights[count] == HIGH) {
      //update lights one by one
      Serial.print("Turning on light number: ");
      Serial.println(count);
      digitalWrite(A4, HIGH);
      delay(200);
      digitalWrite(A4, LOW);
    }
  }
}  

void printValues() {
  for (int j = 0; j <= 7; j++) {
        Serial.print("dialValues1[");
        Serial.print(j);
        Serial.print("]: ");
        Serial.println(dialValues1[j]);
      }
    for (int j = 0; j <= 7; j++) {
      Serial.print("dialValues3[");
      Serial.print(j);
      Serial.print("]: ");
      Serial.println(dialValues3[j]);
    }
  Serial.print("Tempo: ");
  Serial.println(tempoReading);
  Serial.print("Tune: ");
  Serial.println(tuneReading);
  Serial.print("Waveform1: ");
  Serial.println(slide_val);
  Serial.print("Waveform2: ");
  Serial.println(slide2_val);

  Serial.println();
  Serial.println();
  Serial.println();
  Serial.println();
  Serial.println();
  Serial.println();
  Serial.println();

}

// All buttons are pull-downs, so we'll just debounce when the button goes high. 
bool readButton(int buttonPin, int buttonDelay) {
  bool buttonState = digitalRead(buttonPin);  // read the buttonState
  if (buttonState == HIGH) {  
    delay(buttonDelay); // if it's HIGH, then stop reading the button for some amount of time (buttonDelay = some number of milliseconds)
    buttonState = digitalRead(buttonPin); // read the buttonState again 
    if (buttonState == HIGH) {
      return true;  // if it's still HIGH, then yes the button really was pressed. The function will return a value of TRUE
    }
  }
  else {
    return false; // otherwise, then no the button was not pressed. The function will return a value of FALSE
  }
}

void resetAll () {
  Serial.println("Resetting");
  MIDI.sendStop();
    for(int i = 0; i < numKeys; i++) {
      lights[i] = LOW;
    }  
    lightCount = 0;
    delay(500);
}
